@@ -157,16 +157,11 @@ pidgin_connection_report_disconnect_reason (PurpleConnection *gc,
 			if (info->timeout != 0)
 				g_source_remove(info->timeout);
 		}
-		info->timeout = purple_timeout_add(info->delay, do_signon, account);
+		info->timeout = g_timeout_add(info->delay, do_signon, account);
 	} else {
 		if (info != NULL)
 			g_hash_table_remove(auto_reconns, account);
 
-		/*
-		 * TODO: Do we really want to disable the account when it's
-		 * disconnected by wants_to_die?  This happens when you sign
-		 * on from somewhere else, or when you enter an invalid password.
-		 */
 		purple_account_set_enabled(account, PIDGIN_UI, FALSE);
 	}
 
@@ -182,21 +177,23 @@ pidgin_connection_report_disconnect_reason (PurpleConnection *gc,
 	}
 }
 
-static void pidgin_connection_network_connected ()
+static void pidgin_connection_network_connected (void)
 {
-	GList *list = purple_accounts_get_all_active();
+	GList *list, *l;
 	PidginBuddyList *gtkblist = pidgin_blist_get_default_gtk_blist();
 
 	if(gtkblist)
 		pidgin_status_box_set_network_available(PIDGIN_STATUS_BOX(gtkblist->statusbox), TRUE);
 
-	while (list) {
-		PurpleAccount *account = (PurpleAccount*)list->data;
+	l = list = purple_accounts_get_all_active();
+	while (l) {
+		PurpleAccount *account = (PurpleAccount*)l->data;
 		g_hash_table_remove(auto_reconns, account);
 		if (purple_account_is_disconnected(account))
 			do_signon(account);
-		list = list->next;
+		l = l->next;
 	}
+	g_list_free(list);
 }
 
 static gboolean
@@ -206,10 +203,10 @@ disconnect_wait_timeout_cb(gboolean *keep_waiting)
 	return FALSE;
 }
 
-static void pidgin_connection_network_disconnected ()
+static void pidgin_connection_network_disconnected (void)
 {
 #ifdef HAVE_ICD
-	GList *llItr = NULL ;
+	GList *llItr = NULL, *list = NULL;
 	gboolean keep_waiting = TRUE;
 	guint disconnect_wait_timeout_id = 0;
 
@@ -220,43 +217,38 @@ static void pidgin_connection_network_disconnected ()
 
 	/* Wait at most 10 seconds for all accounts to disconenct */
 	disconnect_wait_timeout_id = purple_timeout_add_seconds(10000, (GSourceFunc)disconnect_wait_timeout_cb, &keep_waiting);
-	for (llItr = purple_accounts_get_all_active() ; llItr != NULL && keep_waiting ; llItr = llItr->next) {
+	for (list = llItr = purple_accounts_get_all_active() ; llItr != NULL && keep_waiting ; llItr = llItr->next) {
 		if (!purple_account_is_disconnected((PurpleAccount *)(llItr->data))) {
 			/* No purple_* version of this function yet */
 			g_main_context_iteration(NULL, TRUE);
 			/* Accounts may have changed */
-			llItr = purple_accounts_get_all_active () ;
+			g_list_free(list);
+			list = llItr = purple_accounts_get_all_active();
 		}
 	}
 
+	g_list_free(list);
+
 	if (keep_waiting && disconnect_wait_timeout_id)
 		purple_timeout_remove(disconnect_wait_timeout_id);
 	disconnect_wait_timeout_id = 0;
 	keep_waiting = FALSE;
 #else /* !HAVE_ICD */
-	GList *l = purple_accounts_get_all_active();
+	GList *list, *l;
 	PidginBuddyList *gtkblist = pidgin_blist_get_default_gtk_blist();
-	PurplePluginProtocolInfo *prpl_info = NULL;
-	PurpleConnection *gc = NULL;
-	
+
 	if(gtkblist)
 		pidgin_status_box_set_network_available(PIDGIN_STATUS_BOX(gtkblist->statusbox), FALSE);
 
+	l = list = purple_accounts_get_all_active();
 	while (l) {
 		PurpleAccount *a = (PurpleAccount*)l->data;
 		if (!purple_account_is_disconnected(a)) {
-			gc = purple_account_get_connection(a);
-			if (gc && gc->prpl) 
-				prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(gc->prpl);
-			if (prpl_info) {
-				if (prpl_info->keepalive)
-					prpl_info->keepalive(gc);
-				else
-					purple_account_disconnect(a);
-			}
+			purple_account_disconnect(a);
 		}
 		l = l->next;
 	}
+	g_list_free(list);
 #endif /* HAVE_ICD */
 }
 
